package com.jiuqi.util.chinese;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.util.chinese.ChineseDigit;
import com.jiuqi.util.chinese.ChineseDigitUnit;
import com.jiuqi.util.chinese.NoValidCharException;
import com.jiuqi.util.chinese.NotValidChineseDigitException;
import com.jiuqi.util.chinese.NotValidChineseDigitUnitException;
import com.jiuqi.util.chinese.NotValidChineseNumberException;
import com.jiuqi.util.chinese.StringSplitor;
import java.io.Serializable;
import java.util.ArrayList;

public class ChineseInt implements Cloneable, Serializable {
  private static final char[] INVALID_CHARS = new char[] { ' ', '　' };
  
  private String intValue;
  
  public ChineseInt() {
    this.intValue = "0";
  }
  
  public ChineseInt(long value) {
    this.intValue = String.valueOf(value);
  }
  
  public ChineseInt(String value) {
    if (isIntString(value)) {
      this.intValue = value;
    } else {
      try {
        this.intValue = valueOf(value);
      } catch (Exception exception) {
        this.intValue = "0";
      } 
    } 
  }
  
  public long longValue() {
    return Long.parseLong(this.intValue);
  }
  
  public String chineseValue(boolean showUnit, boolean useBigChar) {
    return parseToChinese(this.intValue, showUnit, useBigChar);
  }
  
  public static String valueOf(String value) throws NotValidChineseNumberException, NoValidCharException {
    boolean negtive = false;
    StringSplitor chars = new StringSplitor(value, INVALID_CHARS);
    char thisChar = chars.getNextValidChar();
    if (thisChar == '(' || thisChar == '（') {
      thisChar = chars.getNextValidChar();
      if (thisChar == '负') {
        thisChar = chars.getNextValidChar();
        if (thisChar == ')' || thisChar == '）') {
          negtive = true;
        } else {
          throw new NotValidChineseNumberException("字符'负'后面的字符无效，必须是')'！");
        } 
      } else {
        throw new NotValidChineseNumberException("字符'('后面的字符无效，必须是'负'！");
      } 
    } 
    if (!negtive)
      chars.rollBack(); 
    boolean hasUnit = false;
    thisChar = chars.getNextValidChar();
    if (ChineseDigit.isValidChineseDigit(thisChar)) {
      thisChar = chars.getNextValidChar();
      if (ChineseDigitUnit.isValidChineseDigitUnit(String.valueOf(thisChar)))
        hasUnit = true; 
      chars.rollBack();
      chars.rollBack();
      long longValue = 0L;
      if (hasUnit) {
        int tempValue = 0;
        while (chars.hasValidChar()) {
          thisChar = chars.getNextValidChar();
          if (ChineseDigit.isValidChineseDigit(thisChar)) {
            try {
              tempValue = ChineseDigit.valueOf(thisChar);
            } catch (NotValidChineseDigitException notValidChineseDigitException) {
              throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
            } 
            if (!chars.hasValidChar())
              longValue += tempValue; 
            continue;
          } 
          if (ChineseDigitUnit.isValidChineseDigitUnit(String.valueOf(thisChar))) {
            if (ChineseDigitUnit.isLargeUnit(thisChar)) {
              longValue += tempValue;
              tempValue = 0;
              try {
                longValue *= Long.parseLong(ChineseDigitUnit.valueOf(String.valueOf(thisChar)));
              } catch (NotValidChineseDigitUnitException notValidChineseDigitUnitException) {
                throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
              } 
              continue;
            } 
            try {
              longValue += tempValue * Long.parseLong(ChineseDigitUnit.valueOf(String.valueOf(thisChar)));
            } catch (NotValidChineseDigitUnitException notValidChineseDigitUnitException) {
              throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
            } 
            tempValue = 0;
            continue;
          } 
          throw new NotValidChineseNumberException("存在非法的字符\"" + thisChar + "\"！");
        } 
      } else {
        StringBuffer stringValue = new StringBuffer();
        while (chars.hasValidChar()) {
          thisChar = chars.getNextValidChar();
          if (ChineseDigit.isValidChineseDigit(thisChar))
            try {
              stringValue.append(ChineseDigit.valueOf(thisChar));
            } catch (NotValidChineseDigitException notValidChineseDigitException) {
              throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
            }  
        } 
        longValue = Long.parseLong(stringValue.toString());
      } 
      if (negtive)
        longValue = -longValue; 
      return String.valueOf(longValue);
    } 
    throw new NotValidChineseNumberException("开头的字符\"" + thisChar + "\"不是数字！");
  }
  
  public static String parseToChinese(String value, boolean showUnit, boolean useBigChar) {
    if (value != null && !value.equals("") && isIntString(value)) {
      boolean negative = false;
      if (value.startsWith("-")) {
        negative = true;
        value = value.substring(1);
      } 
      ArrayList<Integer> digits = new ArrayList();
      for (int i = 0; i < value.length(); i++) {
        int digit = value.charAt(i) - 48;
        if (digit >= 0 && digit <= 9)
          digits.add(new Integer(digit)); 
      } 
      StringBuffer result = new StringBuffer();
      if (!showUnit) {
        for (int j = 0; j < digits.size(); j++) {
          int digit = ((Integer)digits.get(j)).intValue();
          result.append(ChineseDigit.toChineseChar(digit, useBigChar));
        } 
      } else {
        int cursor = digits.size(), smallInt = 0, j = 0;
        while (cursor >= 0) {
          smallInt = getSmallInt(digits, j);
          if (smallInt != 0)
            result.insert(0, String.valueOf(smallIntToChineseString(smallInt, useBigChar)) + 
                ChineseDigitUnit.toChineseUnit(getUnit(j * 4), useBigChar)); 
          cursor -= 4;
          if (smallInt < 1000 && smallInt > 0 && cursor >= 0)
            result.insert(0, ChineseDigit.toChineseChar(0, useBigChar)); 
          j++;
        } 
      } 
      if (negative)
        result.insert(0, "(负)"); 
      return result.toString();
    } 
    throw new IllegalArgumentException("字符串\"" + value + "\"不是合法的阿拉伯数字字符串！");
  }
  
  private static int getSmallInt(ArrayList<Integer> digits, int i) {
    int value = 0, j = 0;
    int cursor = digits.size() - i * 4 - 1;
    while (j < 4 && cursor >= 0) {
      int digit = ((Integer)digits.get(cursor)).intValue();
      value = (int)(value + digit * Math.pow(10.0D, j));
      cursor--;
      j++;
    } 
    return value;
  }
  
  private static String smallIntToChineseString(int value, boolean useBigChar) {
    if (value >= 0 && value < 10000) {
      int[] digits = new int[4];
      int count = 3;
      while (value != 0) {
        digits[count] = value % 10;
        value /= 10;
        count--;
      } 
      StringBuffer result = new StringBuffer();
      boolean allZeroBefore = true;
      boolean zeroExist = false;
      for (int i = 0; i < digits.length; i++) {
        if (digits[i] != 0) {
          result.append(String.valueOf(ChineseDigit.toChineseChar(digits[i], useBigChar)) + 
              ChineseDigitUnit.toChineseUnit(getUnit(3 - i), useBigChar));
          allZeroBefore = false;
          zeroExist = false;
        } else if (!allZeroBefore) {
          boolean allZeroAfter = true;
          for (int j = i + 1; j < 4; j++) {
            if (digits[j] != 0) {
              allZeroAfter = false;
              break;
            } 
          } 
          if (!allZeroAfter && !zeroExist) {
            result.append(ChineseDigit.toChineseChar(0, useBigChar));
            zeroExist = true;
          } 
        } 
      } 
      return result.toString();
    } 
    throw new IllegalArgumentException("只能解析0-9999的整数");
  }
  
  static boolean isIntString(String value) {
    if (value.startsWith("-"))
      value = value.substring(1); 
    return ChineseDigitUnit.isDigitString(value);
  }
  
  private static String getUnit(int i) {
    StringBuffer sb = new StringBuffer("1");
    for (int j = 0; j < i; j++)
      sb.append("0"); 
    return sb.toString();
  }
  
  public Object clone() {
    return new com.jiuqi.util.chinese.ChineseInt(this.intValue);
  }
  
  public boolean equals(Object o) {
    if (o == this)
      return true; 
    if (!(o instanceof com.jiuqi.util.chinese.ChineseInt))
      return false; 
    com.jiuqi.util.chinese.ChineseInt value = (com.jiuqi.util.chinese.ChineseInt)o;
    return this.intValue.equals(value.intValue);
  }
  
  public int hashCode() {
    int result = super.hashCode();
    if (this.intValue != null)
      result ^= this.intValue.hashCode(); 
    return result;
  }
  
  public String toString() {
    return chineseValue(true, false);
  }
}
